package org.apidesign.apifest08.currency;

import java.io.Serializable;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Currency;

/**
 * Default {@link Convertor} implementation. Exchange rate is stored as equivalents. It means if we have USD to CZK convertor and USD1 = CZK17 
 * we store 1 in sourceEquivalent and 17 in destinationEquivalent. This class is immutable.
 * @author lukas
 *
 */
class DefaultConvertor implements ExtendedConvertor, Serializable {

	private static final long serialVersionUID = 6660014633685135034L;

	/**
	 * Equivalent in source currency.
	 */
	private final BigDecimal sourceEquivalent;
	
	/**
	 * Equivalent in destination currency.
	 */
	private final BigDecimal destinationEquivalent;
	
	private final Currency sourceCurrency;
	
	private final Currency destinationCurrency;
	
	public DefaultConvertor(Money sourceEquivalent, Money destinationEquivalent) {
		super();
		if (BigDecimal.ZERO.compareTo(sourceEquivalent.getAmount())==0)
		{
			throw new IllegalArgumentException("Source equivalent amount can not be 0.");
		}
		if (BigDecimal.ZERO.compareTo(destinationEquivalent.getAmount())==0)
		{
			throw new IllegalArgumentException("Destination equivalent amount can not be 0.");
		}
		this.sourceEquivalent = sourceEquivalent.getAmount();
		this.destinationEquivalent = destinationEquivalent.getAmount();
		this.sourceCurrency = sourceEquivalent.getCurrency();
		this.destinationCurrency = destinationEquivalent.getCurrency();
	}
	
	public Money convert(Money amount, Currency destinationCurrency) {
		if (amount==null)
		{
			throw new NullPointerException("Money is null");
		}
		if (destinationCurrency==null)
		{
			throw new NullPointerException("destionationCurrency is null");
		}
		if (!amount.getCurrency().equals(getSourceCurrency()))
		{
			throw new IllegalArgumentException("Can not convert from "+amount.getCurrency()+". Converts between "+getSourceCurrency()+" and "+getDestinationCurrency());
		}
		if (!getDestinationCurrency().equals(destinationCurrency))
		{
			throw new IllegalArgumentException("Can not convert to "+destinationCurrency+". Converts between "+getSourceCurrency()+" and "+getDestinationCurrency());
		}
		BigDecimal sourceAmount = amount.getAmount();
		BigDecimal destinationAmount = sourceAmount.multiply(destinationEquivalent).divide(sourceEquivalent, 2, RoundingMode.HALF_UP);
		return new MoneyImpl(destinationAmount, getDestinationCurrency());
	}
	
	public boolean isConversionSupported(Currency from, Currency to) {
		return sourceCurrency.equals(from) && destinationCurrency.equals(to);
	}

	public BigDecimal getSourceEquivalent() {
		return sourceEquivalent;
	}

	public BigDecimal getDestinationEquivalent() {
		return destinationEquivalent;
	}

	public Currency getSourceCurrency() {
		return sourceCurrency;
	}

	public Currency getDestinationCurrency() {
		return destinationCurrency;
	}
	
	@Override
	public String toString() {
		return getClass().getName()+" converts "+getSourceCurrency()+" to "+getDestinationCurrency()+" "
			+getSourceCurrency()+getSourceEquivalent()+"="+getDestinationCurrency()+getDestinationEquivalent();
	}
}
